/***************************************************************************
 *   Copyright (C) 2007 by Andreas Eckstein                                *
 *   andreas.eckstein@gmx.net                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qregexp.h>
#include <kdesktopfile.h>

#include "findimpl.h"

const QString FindImpl::history_dir_name=QString(".find_history.d");

void FindImpl::applyFilter(ResultList& list, FindURL& url)
{
    if(!url.hasFilter())
        return;

    QString filter = url.filter().replace('.', "\\.");
    filter = filter.replace('*', "[^\\/]*") + "$";
    QRegExp re(filter, false);
    ResultListIterator iter = ResultListIterator(list);

    while(iter.current() != NULL)
    {
        if(re.searchRev((*iter)->path)==-1)
            list.remove(*iter);
        else
            ++iter;
    }
}

static void addAtom(KIO::UDSEntry &entry, unsigned int ID, long l, const QString &s = QString::null)
{
    KIO::UDSAtom atom;
    atom.m_uds = ID;
    atom.m_long = l;
    atom.m_str = s;
    entry.append(atom);
}

FindImpl::FindImpl()
{
    history_dir.setPath(QDir::homeDirPath() + "/" + history_dir_name);
}

void FindImpl::appendToHistory(FindURL& furl)
{
    QFile _history_file(history_dir.path() + "/" + furl.prettyURL().replace('/', "_"));

    if(!history_dir.exists())
        if(!QDir::home().mkdir(history_dir_name))
            return;

    if(_history_file.open(IO_WriteOnly|IO_Append))
    {
            QTextStream stream(&_history_file);
            _history_file.close();
    }
}

void FindImpl::parseHistory(QValueList<KIO::UDSEntry> &list)
{
    history_dir.refresh();

    if(!history_dir.exists())
        return;

    if(history_dir.count()==0)
        return;

    QFileInfoListIterator it(*history_dir.entryInfoList());
    QFileInfo *info;

    while(it.current() != NULL)
    {
        info=*it;
        if(info->isFile())
        {
            FindURL url(info->baseName().replace('_', "/"));
            KIO::UDSEntry entry;
            addAtom(entry, KIO::UDS_URL, 0, url.prettyURL());
            addAtom(entry, KIO::UDS_NAME, 0, url.search());
            addAtom(entry, KIO::UDS_FILE_TYPE, S_IFDIR);
            addAtom(entry, KIO::UDS_MODIFICATION_TIME, info->lastModified().toTime_t(), QString::null);
            addAtom(entry, KIO::UDS_ACCESS_TIME, info->lastRead().toTime_t(), QString::null);
            addAtom(entry, KIO::UDS_CREATION_TIME, info->created().toTime_t(), QString::null);
            addAtom(entry, KIO::UDS_MIME_TYPE, 0, "inode/directory");
            addAtom(entry, KIO::UDS_ICON_NAME, 0, getIcon(url));
            list.append(entry);
        }
        
        ++it;
    }
}

bool FindImpl::getBasename(QString& path, QString& basename)
{
    int i = path.findRev('/');
    if (i > 0)
    {
        basename = path.mid(i+1);
    }
    else
    {
        basename = path;
    }

    return basename != QString::null;
}

static void addSearchEntry(QValueList<KIO::UDSEntry> &list, QString& path, QString& label)
{
    KIO::UDSEntry entry;
        addAtom(entry, KIO::UDS_URL, 0, path);
        addAtom(entry, KIO::UDS_NAME, 0, label);

    list.append(entry);
}

bool FindImpl::listSearch(FindURL& furl, QValueList<KIO::UDSEntry> &list)
{
    uint count=0;
    TrackerFindClient tfclient;
    
    if(!tfclient.search(furl))
        return false;
    
    applyFilter(tfclient.results, furl);
    
    ResultListIterator iter = ResultListIterator(tfclient.results);

    for(; iter.current() != NULL; ++iter)
    {
        QString path = "file://" + (*iter)->path;
        QString basename;
        if(getBasename(path, basename))
            addSearchEntry(list, path, basename);
        count++;
    }

    if(count>0)
    //We have at least one result, so we'll append the URL to the history file
        appendToHistory(furl);

    return true;
}

bool FindImpl::listSearches(QValueList<KIO::UDSEntry> &list)
{
    parseHistory(list);

    return true;
}

void FindImpl::createTopLevelEntry(KIO::UDSEntry &entry) const
{
    entry.clear();
    addAtom(entry, KIO::UDS_NAME, 0, ".");
    addAtom(entry, KIO::UDS_FILE_TYPE, S_IFDIR);
    addAtom(entry, KIO::UDS_ACCESS, 0555);
    addAtom(entry, KIO::UDS_MIME_TYPE, 0, "inode/directory");
    addAtom(entry, KIO::UDS_ICON_NAME, 0, "find");
    addAtom(entry, KIO::UDS_USER, 0, "root");
    addAtom(entry, KIO::UDS_GROUP, 0, "root");
}

void FindImpl::createSearchSecondLevelEntry(KIO::UDSEntry &entry, QString searchstring) const
{
    entry.clear();
    addAtom(entry, KIO::UDS_NAME, 0, ".");
    addAtom(entry, KIO::UDS_FILE_TYPE, S_IFDIR);
    addAtom(entry, KIO::UDS_ACCESS, 0555);
    addAtom(entry, KIO::UDS_MIME_TYPE, 0, "inode/directory");
    addAtom(entry, KIO::UDS_ICON_NAME, 0, "kfind");
    addAtom(entry, KIO::UDS_USER, 0, "root");
    addAtom(entry, KIO::UDS_GROUP, 0, "root");
}

const char* FindImpl::getIcon(FindURL &url)
{
    ServiceType type = TrackerFindClient::getServiceType(url);
    return TrackerFindClient::getServiceIcon(type);
}
